@@ -225,15 +225,9 @@ mlir::Value createSubroutineCall(AbstractConverter &converter,
                                  SymMap &symMap, StatementContext &stmtCtx,
                                  bool isUserDefAssignment);
 
-// Attribute for an alloca that is a trivial adaptor for converting a value to
-// pass-by-ref semantics for a VALUE parameter. The optimizer may be able to
-// eliminate these.
-inline mlir::NamedAttribute getAdaptToByRefAttr(fir::FirOpBuilder &builder) {
-  return {mlir::StringAttr::get(builder.getContext(),
-                                fir::getAdaptToByRefAttrName()),
-          builder.getUnitAttr()};
-}
-
+mlir::Value addCrayPointerInst(mlir::Location loc, fir::FirOpBuilder &builder,
+                               mlir::Value ptrVal, mlir::Type ptrTy,
+                               mlir::Type pteTy);
 } // namespace Fortran::lower
 
 #endif // FORTRAN_LOWER_CONVERTEXPR_H
